#This is the Final Project for Peter Chung for Chemical Engineering 210D.

#import python modules
import numpy as np
import time

import sys
import os
import exflib
import exflib2
import exflib3
import atomwrite

#========== GLOBAL VARIABLES ==========
#distance cutoff for pairwise interactions
Cut = 2.5

def CalcEnergy(Pos):
    """Wrapper for energy calculation.
Input:
    Pos: (N,3) position array
Output:
    PEnergy: potential energy, float
"""
    global M, L, Cut
    return ex4lib.calcenergy(Pos, M, L, Cut)


def InitPositions(N):
    #Returns an array of initial positions of each atom, in a line
    #N: number of atoms
    #Output:
    #Pos: (N,3) array of positions

    #make the position array
    Pos = np.zeros((N,3), float)
    #compute integer grid # of locations for cubic lattice
    t = 0
    c = 0
    for x in range(N):
        Pos[t] = Pos[t] + [c,0,0]
        t = t + 1
        c = c + 1
    return Pos
"""
def InitPositions(N, L):
#    Returns an array of initial positions of each atom,
#placed on a cubic lattice for convenience.
#Input:
#    N: number of atoms
#    L: box length
#Output:
#    Pos: (N,3) array of positions


    #make the position array
    Pos = np.zeros((N,3), float)
    #compute integer grid # of locations for cubic lattice
    NLat = int(N**(1./3.) + 1.)
    LatSpac = L / NLat
    #make an array of lattice sites
    r = LatSpac * np.arange(NLat, dtype=float) - 0.5*L
    #loop through x, y, z positions in lattice until done
    #for every atom in the system
    i = 0
    for x in r:
        for y in r:
            for z in r:
                Pos[i] = np.array([x,y,z], float)
                #add a random offset to help initial minimization
                Offset = 0.1 * LatSpac * (np.random.rand(3) - 0.5)
                Pos[i] = Pos[i] + Offset
                i += 1
                #if done placing atoms, return
                if i >= N:
                    return Pos
    return Pos
"""
def InitCharges(N):
    """Returns an array of initial positions of each atom, in a line
    N: number of atoms
Output:
    Pos: (N,3) array of positions
"""
    #make the position array
    q = np.zeros((N,1), float)
    #compute integer grid # of locations for cubic lattice
    t = 0
    for x in range(N):
        q[t] = q[t] + [1. * np.random.rand()]
        t = t+1
    return q


def Run(dx):
    #set the init box width, number of particles, temperature, and timestep
    Temp = 1.0
    dt = 0.001
    rc=2.5
    Dim = 3
    M = 20
    Pos = InitPositions(M)
    q = InitCharges(M)
    Pos0 = Pos
    i = 0
    j = 0
    Pdb = atomwrite.pdbfile("anim.pdb", Compressed = False)
    while i < 2000:
        while j < 2000:
            MoveAtom = np.random.randint(len(Pos))
            OldPos = Pos[MoveAtom,:].copy()
            Old = exflib3.calcenergy(Pos, q, Cut, Dim)
            Pos[MoveAtom,:] = OldPos + 2 * dx * np.random.rand(len(Pos[MoveAtom,:])) - dx
            New = exflib3.calcenergy(Pos, q, Cut, Dim)
            Delta = Old - New
            R = np.exp(Delta)
            Decision = np.minimum(1, R)
            if Decision < np.random.rand():
                Pos[MoveAtom,:] = OldPos
            j = j + 1
        i = i + 1
        j = 0
        Pdb.write(Pos)
    Pdb.close()
    r1 = np.sqrt(sum(sum((Pos-Pos.mean(axis=0))**2))/(len(Pos)))
    r1std = np.std(Pos)
    return r1, r1std
    
"""
    M = 8
    Pos = InitPositions(M)
    q = InitCharges(M)
    i = 0
    j = 0
    while i < 200000:
        while j < M:
            MoveAtom = np.random.randint(len(Pos))
            OldPos = Pos[MoveAtom,:].copy()
            Old = exflib3.calcenergy(Pos, q, Cut, Dim)
            Pos[MoveAtom,:] = OldPos + 2 * dx * np.random.rand(len(Pos[MoveAtom,:])) - dx
            New = exflib3.calcenergy(Pos, q, Cut, Dim)
            Delta = Old - New
            R = np.exp(Delta)
            Decision = np.minimum(1, R)
            if Decision < np.random.rand():
                Pos[MoveAtom,:] = OldPos
            j = j + 1
        i = i + 1
        j = 0
    r2 = np.sqrt(sum(sum((Pos-Pos.mean(axis=0))**2))/(len(Pos)))
    r2std = np.std(Pos)
    M = 12
    Pos = InitPositions(M)
    q = InitCharges(M)
    i = 0
    j = 0
    while i < 200000:
        while j < M:
            MoveAtom = np.random.randint(len(Pos))
            OldPos = Pos[MoveAtom,:].copy()
            Old = exflib3.calcenergy(Pos, q, Cut, Dim)
            Pos[MoveAtom,:] = OldPos + 2 * dx * np.random.rand(len(Pos[MoveAtom,:])) - dx
            New = exflib3.calcenergy(Pos, q, Cut, Dim)
            Delta = Old - New
            R = np.exp(Delta)
            Decision = np.minimum(1, R)
            if Decision < np.random.rand():
                Pos[MoveAtom,:] = OldPos
            j = j + 1
        i = i + 1
        j = 0
    r3 = np.sqrt(sum(sum((Pos-Pos.mean(axis=0))**2))/(len(Pos)))
    r3std = np.std(Pos)
    M = 16
    Pos = InitPositions(M)
    q = InitCharges(M)
    i = 0
    j = 0
    while i < 200000:
        while j < M:
            MoveAtom = np.random.randint(len(Pos))
            OldPos = Pos[MoveAtom,:].copy()
            Old = exflib3.calcenergy(Pos, q, Cut, Dim)
            Pos[MoveAtom,:] = OldPos + 2 * dx * np.random.rand(len(Pos[MoveAtom,:])) - dx
            New = exflib3.calcenergy(Pos, q, Cut, Dim)
            Delta = Old - New
            R = np.exp(Delta)
            Decision = np.minimum(1, R)
            if Decision < np.random.rand():
                Pos[MoveAtom,:] = OldPos
            j = j + 1
        i = i + 1
        j = 0
    r4 = np.sqrt(sum(sum((Pos-Pos.mean(axis=0))**2))/(len(Pos)))
    r4std = np.std(Pos)
    M = 20
    Pos = InitPositions(M)
    q = InitCharges(M)
    i = 0
    j = 0
    while i < 200000:
        while j < M:
            MoveAtom = np.random.randint(len(Pos))
            OldPos = Pos[MoveAtom,:].copy()
            Old = exflib3.calcenergy(Pos, q, Cut, Dim)
            Pos[MoveAtom,:] = OldPos + 2 * dx * np.random.rand(len(Pos[MoveAtom,:])) - dx
            New = exflib3.calcenergy(Pos, q, Cut, Dim)
            Delta = Old - New
            R = np.exp(Delta)
            Decision = np.minimum(1, R)
            if Decision < np.random.rand():
                Pos[MoveAtom,:] = OldPos
            j = j + 1
        i = i + 1
        j = 0
    r5 = np.sqrt(sum(sum((Pos-Pos.mean(axis=0))**2))/(len(Pos)))
    r5std = np.std(Pos)
    M = 24
    Pos = InitPositions(M)
    q = InitCharges(M)
    i = 0
    j = 0
    while i < 200000:
        while j < M:
            MoveAtom = np.random.randint(len(Pos))
            OldPos = Pos[MoveAtom,:].copy()
            Old = exflib3.calcenergy(Pos, q, Cut, Dim)
            Pos[MoveAtom,:] = OldPos + 2 * dx * np.random.rand(len(Pos[MoveAtom,:])) - dx
            New = exflib3.calcenergy(Pos, q, Cut, Dim)
            Delta = Old - New
            R = np.exp(Delta)
            Decision = np.minimum(1, R)
            if Decision < np.random.rand():
                Pos[MoveAtom,:] = OldPos
            j = j + 1
        i = i + 1
        j = 0
    r6 = np.sqrt(sum(sum((Pos-Pos.mean(axis=0))**2))/(len(Pos)))
    r6std = np.std(Pos)
    return r1, r2, r3, r4, r5, r6, r1std, r2std, r3std, r4std, r5std, r6std
"""